/* ******************************************************************************
   Hello World Serial LCD Interface

   This is a "sender" sketch.
   It sends LCD commands via Serial to a Serial LCD or another Arduino with a physical LCD.
   Local keys send different text / actions to the receiver LCD

   Hardware:
   - external LCD with Serial interface like Sureeno (or another Arduino with a physical LCD and a RX sketch)
   - resistor network keys connected to A0
   - Using MAX485 to send data to RS485 lines

   Variants:
   - Sureeno http://surenoo.tech/download/01_SLC/SLC1602F.pdf

   Important:

   by noiasca
   2022-12-27

 ****************************************************************************** */

#include <NoiascaLiquidCrystal.h>      // download library from https://werner.rothschopf.net/202009_arduino_liquid_crystal_intro.htm  
#include <NoiascaHW/lcd_serial.h>      // include the proper HW interface for a serial remote display

/* *******************************************************
   Serial Interface
 * **************************************************** */
// if you don't have enough HW Serial (i.e. on an UNO)
// you are forced to use SoftwareSerial or AltSoftSerial
#include <SoftwareSerial.h>
constexpr byte rxPin = 2;
constexpr byte txPin = 3;
SoftwareSerial mySerial(rxPin, txPin);               // RXpin, TXpin

// If you don't need debugging output, you can use
// a reference to Serial
//HardwareSerial &mySerial = Serial;

// On a Mega you can simply use
// a Reference to an existing HW Serial:
//HardwareSerial &mySerial = Serial3;

LiquidCrystal_Serial_base lcd(mySerial);                   // create lcd object with 16 cols 2 rows for a Sureeno Serial display

constexpr uint32_t interval = 2000;
constexpr byte enablePin = 11;          // The GPIO used to control the MAX485 TX pin.  (5 on MEGA, 11 on UNO)
constexpr uint8_t keyPin = A0;          // an analog pin with several keys

void preTransmission()
{
  digitalWrite(enablePin, HIGH);
  delay(5);
}

void postTransmission()
{
  //mySerial.flush();        // wait till everything was sent
  delay(1);
  digitalWrite(enablePin, LOW);
}

void key(uint8_t key)
{
  Serial.print(F("key press on RX ")); Serial.println(key);
}

uint8_t keyRead()
{
  uint16_t keyAdc = analogRead(keyPin); // read the value from the sensor
  //keyAdc = analogRead(keyPin);          // measure twice
  uint8_t key = 0;
  //Serial.println(keyAdc);        // this is just for testing the ADC values
  if (keyAdc > 1000)
  {
    ;                              // nothing
  }
  else if (keyAdc < 50) key = 1;   // 50 right
  else if (keyAdc < 230) key = 2;  // 195 up
  else if (keyAdc < 405) key = 3;  // 380 down
  else if (keyAdc < 600) key = 4;  // 555 left
  else if (keyAdc < 790) key = 5;  // 790 select
  return key;
}

// read local keys and perform a remote action on the receiver
void keyHandle()
{
  static uint32_t previousMillis = 0;
  uint32_t currentMillis = millis();
  static uint8_t previousKey = 0;
  if (currentMillis - previousMillis > 100)
  {
    previousMillis = currentMillis;
    uint8_t key = keyRead();
    if (key != previousKey)
    {
      switch (key)
      {
        case 0:
         lcd.clear();
         lcd.print(F("idle mode"));
         break;
        case 1: 
         lcd.clear();
         lcd.print(F("key 1 on TX"));
         break;
        case 2: 
         lcd.clear();
         lcd.print(F("key 2 on TX"));
         break;
        case 3: 
         lcd.clear();
         lcd.print(F("key 3 on TX"));
         break;
        case 4: 
         lcd.clear();
         lcd.print(F("key 4 on TX"));
         break;
        case 5: 
         lcd.clear();
         lcd.print(F("key 5 on TX"));
         break;
      }
      previousKey = key;
    }
  }
}


// print some values to the Serial LCD
void test()
{
  static uint32_t previousMillis = 0;
  if (millis() - previousMillis > interval)
  {
    previousMillis = millis();
    lcd.setCursor(0, 1);
    lcd.print(millis() / 1000);
    lcd.print(' ');
    lcd.write(random(8)); // one of the 8 special characters
    lcd.print(F("           "));
  }
}

void setup()
{
  Serial.begin(9600);                            // start Serial (used for debugging)
  mySerial.begin(9600);                          // start the Serial used for LCD transmission
  Serial.println(F("\nSerial LCD Sender"));
  pinMode(enablePin, OUTPUT);                    // needed for RS485 enable line
  lcd.setPreTransmission(preTransmission);       // declare callback for pre transmission
  lcd.setPostTransmission(postTransmission);     // declare callback for post transmission
  lcd.setKey(key);                               // declare callback on key press events
  lcd.begin();                                   // initialize the LCD
  //lcd.clear();                                 // example how to clear
  lcd.backlight();                               // turn on backlight
  //lcd.cursor();
  //lcd.noCursor();
  //lcd.blink();
  //lcd.noBlink();
  //lcd.off();
  //lcd.on();
  lcd.setCursor(0, 0);                           // use column - row (!)
  lcd.print(F("Hello world!"));
  lcd.setCursor(0, 1);
  lcd.print(F("2nd row"));
  //lcd.print("αβμΣ°÷∞←→äöüßÄÖÜ");               // show some special character entered in UTF-8
}

void loop()
{
  //test();        // if you disable the sending of test messages you will see the effects of the LCD backlight handling
  keyHandle();
  lcd.parse();     // enable readout of Server pushes key strokes - NOK
}
