/*******************************************************************************
   Convert for 3rd Party LCD Library

   This example shows how the use the build in UTF-8 converter from NoiascaLiquidCrystal
   for other (3rd party) LCD libraries.
   So if you need to use another LCD library but you want to use the UTF-8 support,
   use this pattern. For this example I'm using the Adafruit LCD Library.

   libraries
   - NoiascaLiquidCrystal.h    (this library)
   - Adafruit_LiquidCrystal.h  by Adafruit
   by noiasca
   2021-02-03  OK (with patched Adafruit Library)

 *******************************************************************************/

#include <NoiascaLiquidCrystal.h>      // download library from https://werner.rothschopf.net/202009_arduino_liquid_crystal_intro.htm

#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Max72xxPanel.h>

int pinCS = 10; // Attach CS to this pin, DIN to MOSI and CLK to SCK (cf http://arduino.cc/en/Reference/SPI )
int numberOfHorizontalDisplays = 4;
int numberOfVerticalDisplays = 1;

String tape = "Arduino";
int wait = 20; // In milliseconds
int spacer = 1;
int width = 5 + spacer; // The font width is 5 pixels

// now lets create a very specific display object to be used with the 3rd party libray
// the new class will inherit all variables and methods from the 3rd party library.
// It will be the parent class - and will be enriched with one variable and a modified member function

//Max72xxPanel matrix = Max72xxPanel(pinCS, numberOfHorizontalDisplays, numberOfVerticalDisplays);



class MyPanel : public Max72xxPanel              // class shoud inherit from 3rd party class
{
  protected:
    uint32_t special = 0;                        // buffer to store incomming special characters

  public :
    MyPanel(byte csPin, byte hDisplays = 1, byte vDisplays = 1) :
      Max72xxPanel(csPin, hDisplays, vDisplays),            // the default constructor must call the constructor of the parent class
      funcPtr(convert)              // function pointer to default converter
    {}                              // MyLCD constructor calls the 3rd party constructor

    using CallBack = uint8_t (*)(uint32_t &special, uint8_t &value);
    CallBack funcPtr;

    // next we need an adoption of the write() method
    // write() - process data character byte to lcd
    // it returns number of bytes successfully PROCESSED. Either printed to the device or stored to special
    // i.e. 1 if success or 0 if no character was processed (error)
    size_t write(uint8_t value)
    {
      switch (funcPtr (special, value))
      {
        case NOPRINT :       // don't print
          break;
        case ADDE :          // print an additional e after character
          Adafruit_GFX::write('e');
          Adafruit_GFX::write(value);
          break;
        default :            // includes PRINT: just print
          Adafruit_GFX::write(value);
          break;
      }
      return (1);            // we always assume success (either print to display or store to special character
    }

    void write()
    {
      Max72xxPanel::write();
    }

};

MyPanel matrix(pinCS, numberOfHorizontalDisplays, numberOfVerticalDisplays);     // now create the object based on the new class

void setup()
{
  matrix.setIntensity(7);
  matrix.print("ÄäÖöÜüß°");
  matrix.write(); // Send bitmap to display

}

void loop()
{}
